//Version 1.0.0.15

//ENVIRONMENT
exports.Environment = {
	osname : Ti.Platform.osname,
	version : Ti.Platform.version,
	height : Ti.Platform.displayCaps.platformHeight,
	width : Ti.Platform.displayCaps.platformWidth,
	isAndroid : Ti.Platform.osname === 'android',
	isApple : (Ti.Platform.osname === 'iphone' || Ti.Platform.osname === 'ipad' || Ti.Platform.osname === 'ipod'),
	isIPhone : Ti.Platform.osname === 'iphone',
	isIPhone3 : Ti.Platform.osname === 'iphone' && (Ti.Platform.displayCaps.density == 'high') ? ((Ti.Platform.displayCaps.platformHeight == 480) ? false : false) : true,
	isIPhone4 : Ti.Platform.osname === 'iphone' && (Ti.Platform.displayCaps.density == 'high') ? ((Ti.Platform.displayCaps.platformHeight == 480) ? true : false) : false,
	isIPhone5 : Ti.Platform.osname === 'iphone' && (Ti.Platform.displayCaps.density == 'high') ? ((Ti.Platform.displayCaps.platformHeight == 480) ? false : true) : false,
	isIPad : Ti.Platform.osname === 'ipad',
	isIPod : Ti.Platform.osname === 'ipod',
	isTablet : Ti.Platform.osname === 'ipad' || (Ti.Platform.osname === 'android' && (Ti.Platform.displayCaps.platformWidth > 899 || Ti.Platform.displayCaps.platformHeight > 899)),
	isLandscape : function(orient) {
		orient = orient || Ti.UI.orientation;
		return orient == Ti.UI.LANDSCAPE_LEFT || orient == Ti.UI.LANDSCAPE_RIGHT;
	},
	isPortrait : function(orient) {
		orient = orient || Ti.UI.orientation;
		return orient == Ti.UI.PORTRAIT || orient == Ti.UI.UPSIDE_PORTRAIT;
	},
	isiOS7Plus : function() {
		// iOS-specific test
		if (exports.Environment.isApple) {
			var version = Titanium.Platform.version.split(".");
			var major = parseInt(version[0], 10);

			// Can only test this support on a 3.2+ device
			if (major >= 7) {
				return true;
			}
		}
		return false;
	}
};
//UTILITY
exports.Utility = {
	extend : function(options, defaults) {
		if (options == null)
			return defaults;
		else {
			for (name in defaults ) {
				if (options[name] == undefined)
					options[name] = defaults[name];
			}
			return options;
		}
	}
};
//MATH
exports.Math = {
	Random : function(minVal, maxVal) {
		var randVal = minVal + (Math.random() * (maxVal - minVal));
		return Math.round(randVal);
	},
	RoundNumber : function(num, dec) {
		return Math.round(num * Math.pow(10, dec)) / Math.pow(10, dec);
	}
};
//DIALOG
exports.Dialog = {
	Confirm : function(title, message, onSuccess, onCancel) {
		var alert = Titanium.UI.createAlertDialog({
			title : title,
			message : message,
			buttonNames : ['Si', 'No'],
			cancel : 1
		});
		alert.addEventListener('click', function(e) {
			//Clicked cancel, first check is for iphone, second for android
			if (e.cancel === e.index || e.cancel === true) {
				if (onCancel != null)
					onCancel();
			}
			switch (e.index) {
				case 0:
					if (onSuccess != null)
						onSuccess();
					break;
				default:
					break;
			}
		});
		alert.show();
	},
	Alert : function(title, message, onSuccess) {
		var alert = Titanium.UI.createAlertDialog({
			title : title,
			message : message,
			buttonNames : ['OK']
		});
		alert.addEventListener('click', function(e) {
			//Clicked cancel, first check is for iphone, second for android
			if (e.cancel === e.index || e.cancel === true) {
				return;
			}
			switch (e.index) {
				case 0:
					if (onSuccess != null)
						onSuccess();
					break;
				case 1:
					break;
				default:
					break;
			}
		});
		alert.show();
	},
	Error : function(message) {
		var alert = Titanium.UI.createAlertDialog({
			title : '',
			message : message,
			buttonNames : ['OK']
		});
		alert.show();
	}
};
//ARRAY
exports.Array = {
	Filter : function(array, property, pattern) {
		var result = [];

		for (var i = 0; i < array.length; i++) {
			if (exports.Date.IsDate(array[i][property])) {
				if (exports.Date.Compare(array[i][property], pattern) == 0) {
					result.push(array[i]);
				}
			} else {
				if (array[i][property] == pattern) {
					result.push(array[i]);
				}
			}
		}

		return result;
	},
	FilterLike : function(array, property, pattern) {
		var searchLen = pattern.length;
		array.sort();
		var result = [];
		for (var index = 0, len = array.length; index < len; index++) {
			if (array[index][property].substring(0, searchLen).toUpperCase() === pattern.toUpperCase()) {
				result.push(array[index]);
			}
		}
		return result;
	},
	Sort : function(array, property, direction) {
		direction = direction || 'ASC';

		return array.sort(function(a, b) {
			var x, y;
			//Verifichiamo il tipo
			if ( typeof a[property] != 'string') {
				x = a[property];
				y = b[property];
			} else {
				x = a[property].toLowerCase();
				y = b[property].toLowerCase();
			}
			if (direction == 'ASC')
				return ((x < y) ? -1 : ((x > y) ? 1 : 0));
			else
				return ((x < y) ? 1 : ((x > y) ? -1 : 0));
		});
	},
	TableViewIndex : function(array, property) {
		var index = [];

		var lastChar = '';
		var firstChar = '';
		for (var i = 0; i < array.length; i++) {
			firstChar = array[i][property].substring(0, 1).toString().toUpperCase();
			if (firstChar != lastChar) {
				lastChar = firstChar;
				index.push({
					title : lastChar,
					index : i
				});
			}
		}

		return exports.Array.Sort(index, 'title');
	},
	Pivot : function(array, rowProperty, columnProperty, valueProperty) {
		var result = [];
		var item, filtered;

		for (var i = 0; i < array.length; i++) {
			//verifichiamo se la rowProperty è già presente
			filtered = exports.Array.Filter(result, rowProperty, array[i][rowProperty]);
			if (filtered.length > 0) {
				item = filtered[0];
			} else {
				var item = {};
				item[rowProperty] = array[i][rowProperty];
				result.push(item);
			}
			item[array[i][columnProperty]] = array[i][valueProperty];
		}

		return result;
	},
	DistinctValueProperty : function(array, rowProperty) {
		var result = [];
		var item, filtered;

		for (var i = 0; i < array.length; i++) {
			//verifichiamo se la rowProperty è già presente
			filtered = exports.Array.Filter(result, rowProperty, array[i][rowProperty]);
			if (filtered.length == 0) {
				var item = {};
				item[rowProperty] = array[i][rowProperty];
				result.push(item);
			}
		}

		return result;
	},
	Normalize : function(array, keys, rowProperty) {
		for (var i = 0; i < array.length; i++) {
			for (var j = 0; j < keys.length; j++) {
				if (!array[i][keys[j][rowProperty]])
					array[i][keys[j][rowProperty]] = 0;
			}
		}

		return array;
	}
};
//DATE
exports.Date = {
	AddHour : function(date, hours) {
		date.setHours(date.getHours() + hours);
		return date;
	},
	AddDay : function(date, days) {
		date.setDate(date.getDate() + days);
		return date;
	},
	AddMonth : function(date, months) {
		date.setMonth(date.getMonth() + months);
		return date;
	},
	AddMinute : function(date, minutes) {
		date.setMinutes(date.getMinutes() + minutes);
		return date;
	},
	Format : function(date, format) {
		var o = {
			"M+" : date.getMonth() + 1, //month
			"d+" : date.getDate(), //day
			"h+" : date.getHours(), //hour
			"m+" : date.getMinutes(), //minute
			"s+" : date.getSeconds(), //second
			"q+" : Math.floor((date.getMonth() + 3) / 3), //quarter
			"S" : date.getMilliseconds() //millisecond
		};

		if (/(y+)/.test(format))
			format = format.replace(RegExp.$1, (date.getFullYear() + "").substr(4 - RegExp.$1.length));
		for (var k in o)
		if (new RegExp("(" + k + ")").test(format))
			format = format.replace(RegExp.$1, RegExp.$1.length == 1 ? o[k] : ("00" + o[k]).substr(("" + o[k]).length));
		return format;
	},
	GetFromString : function(text) {
		text = text.substring(0, 19).replace(/-/g, '/').replace(/T/g, ' ');
		return new Date(text);
	},
	Clone : function(date) {
		var y = date.getFullYear();
		var M = date.getMonth();
		//month
		var d = date.getDate();
		//day
		var h = date.getHours();
		//hour
		var m = date.getMinutes();
		//minute
		var s = date.getSeconds();

		return new Date(y, M, d, h, m, s);
	},
	IsDate : function(date) {
		alert(date);
		if (date != null)
			return Object.prototype.toString.call(date) === '[object Date]';
		else
			return false;
	},
	Compare : function(a, b) {
		/* Compare the current date against another date.
		 *
		 * @param b  {Date} the other date
		 * @returns   -1 : if a < b
		 *             0 : if a === b
		 *             1 : if a > b
		 *            NaN : if a or b is an illegal date
		 */
		return (isFinite(a.valueOf()) && isFinite(b.valueOf()) ? (a > b) - (a < b) : NaN
		);
	},
	FromInteger : function(number) {
		return new Date(number * 1000);
	},
	FromDB : function(text) {
		if (text == '')
			return null;
	
		var y = parseInt(text.substr(0, 4));
		var M = parseInt(text.substr(4, 2));
		var d = parseInt(text.substr(6, 2));
		var h = parseInt(text.substr(8, 2));
		var m = parseInt(text.substr(10, 2));
		var s = parseInt(text.substr(12, 2));
				
		return new Date(y, M - 1, d, h, m, s);
	},
	ToDB : function(date) {
		if (!date)
			return '';

		var y = date.getFullYear();

		var M = date.getMonth() + 1;
		if (M < 10)
			M = '0' + M;

		var d = date.getDate();
		if (d < 10)
			d = '0' + d;

		var h = date.getHours();
		if (h < 10)
			h = '0' + h;

		var m = date.getMinutes();
		if (m < 10)
			m = '0' + m;

		var s = date.getSeconds();
		if (s < 10)
			s = '0' + s;

		return y.toString() + M.toString() + d.toString() + h.toString() + m.toString() + s.toString();
	}
};
//STRING
exports.String = {
	IsNumber : function(n) {
		return !isNaN(parseFloat(n)) && isFinite(n);
	},
	Replace : function(expression, find, replacement) {
		var outString = expression;
		while (true) {
			var idx = outString.indexOf(find);
			if (idx == -1) {
				break;
			}
			outString = outString.substring(0, idx) + replacement + outString.substring(idx + find.length);
		}
		return outString;
	}
};
//IO
exports.IO = {
	FormatFileSize : function(filesize) {
		if (filesize >= 1099511627776) {
			return exports.Math.RoundNumber(filesize / 1099511627776, 2) + ' Tb';
		}
		if (filesize >= 1073741824) {
			return exports.Math.RoundNumber(filesize / 1073741824, 2) + ' Gb';
		}
		if (filesize >= 1048576) {
			return exports.Math.RoundNumber(filesize / 1048576, 2) + ' Mb';
		}
		if (filesize >= 1024) {
			return exports.Math.RoundNumber(filesize / 1024, 0) + ' Kb';
		}
		return filesize + ' byte';
	}
};
//WINDOWS
exports.Windows = {
	PopUp : {
		GetOptions : function() {
			return {
				backgroundColor : '#f7f7f7',
				borderColor : 'silver',
				borderWidth : 1,
				width : 300,
				height : 210,
				title : '',
				color : '#000',
				layout : ''
			};
		},
		Create : function(options, onSuccess, onCancel) {
			var defaults = exports.Windows.PopUp.GetOptions();
			options = exports.Utility.extend(options, defaults);

			var win = Ti.UI.createWindow();

			var backView = Ti.UI.createView({
				backgroundColor : '#000',
				opacity : 0.5
			});
			win.add(backView);

			var scroll = Ti.UI.createScrollView();
			win.add(scroll);

			var view = Ti.UI.createScrollView({
				backgroundColor : options.backgroundColor,
				width : options.width,
				borderColor : options.borderColor,
				borderWidth : options.borderWidth,
				layout : 'vertical'
			});
			scroll.add(view);

			var header = Ti.UI.createView({
				top : 0,
				height : 30
			});
			view.add(header);

			var lblTitle = Ti.UI.createLabel({
				text : options.title,
				color : options.color,
				top : 7,
				left : 7,
				font : {
					fontSize : 16,
					fontWeight : 'bold'
				},
				textAlign : 'center'
			});
			header.add(lblTitle);

			var container = Ti.UI.createView({
				layout : options.layout
			});
			view.add(container);

			var footer = Ti.UI.createView({
				bottom : 0,
				height : 50
			});
			view.add(footer);

			view.height = options.height + header.height + footer.height;
			container.height = options.height;

			var btnCancel = Ti.UI.createButton({
				bottom : 10,
				left : 10,
				width : options.width / 2,
				title : 'Chiudi'
			});
			if (exports.Environment.isAndroid)
				btnCancel.height = 50;
			footer.add(btnCancel);
			btnCancel.addEventListener('click', function() {
				if (onCancel != null)
					onCancel();
				win.close();
			});

			var btnDone = Ti.UI.createButton({
				bottom : 10,
				right : 10,
				width : options.width / 2,
				title : 'Conferma'
			});
			if (exports.Environment.isAndroid)
				btnDone.height = 50;
			footer.add(btnDone);
			btnDone.addEventListener('click', function() {
				if (onSuccess != null) {
					if (onSuccess())
						win.close();
				} else {
					win.close();
				}
			});

			//EVENTS
			win.getContainer = function() {
				return container;
			};

			return win;
		}
	},
	DatePickerInterval : function(minDate, maxDate) {
		var selectDate;

		var options = {
			title : 'Selezionare la data...',
			width : 324
		};

		var onSuccess = function() {
			winPopUp.fireEvent('toolkit_DatePickerIntervalSelected', {
				date : selectDate
			});
			return true;
		};
		var winPopUp = exports.Windows.PopUp.Create(options, onSuccess, null);
		var view = winPopUp.getContainer();

		var picker = Ti.UI.createPicker({
			type : Ti.UI.PICKER_TYPE_DATE,
			selectionIndicator : true,
			minDate : minDate || null,
			maxDate : maxDate || null
		});
		picker.addEventListener("change", function(e) {
			selectDate = e.value;
		});
		view.add(picker);

		winPopUp.setValue = function(currentDate) {
			selectDate = currentDate;
			picker.value = currentDate;
		};

		return winPopUp;
	}
};
//CONTROLS
exports.Controls = {
	RemoveAllChildrens : function(object) {
		var childrenCount = object.children.length;
		for ( i = 0; i < childrenCount; i++) {
			object.remove(object.children[((childrenCount - 1) - i)]);
		}
	},
	Selector : {
		GetOptions : function() {
			return {
				height : '100%',
				width : '100%',
				left : 0,
				top : 0,
				backgroundColor : 'black',
				color : 'silver',
				backgroundSelectedColor : 'white',
				index : 0,
				font : {
					fontSize : 10
				},
				baselineSmall : true,
				changeColorTextOnSelect : true
			};
		},
		Create : function(items, style) {
			var defaults = exports.Controls.Selector.GetOptions();
			style = exports.Utility.extend(style, defaults);

			var lblWidth = parseInt(100 / items.length) + '%';

			var view = Ti.UI.createView({
				height : style.height,
				width : style.width,
				left : style.left,
				top : style.top,
				backgroundColor : style.backgroundColor
			});

			var viewItems = Ti.UI.createView({
				top : 0,
				layout : 'horizontal',
				height : style.height - 5
			});
			view.add(viewItems);
			view.index = 0;

			var baseline = Ti.UI.createView({
				backgroundColor : style.color,
				height : (style.baselineSmall ? 1 : 3),
				bottom : (style.baselineSmall ? 2 : 1)
			});
			view.add(baseline);

			var cursor = Ti.UI.createView({
				bottom : 1,
				left : 0,
				height : 3,
				width : lblWidth,
				backgroundColor : style.backgroundSelectedColor
			});
			view.add(cursor);

			view.setPosition = function(index) {
				if (items.length > 0) {
					for (var i = 0; i < viewItems.children.length; i++) {
						viewItems.children[i].color = style.color;
					}
					if (style.changeColorTextOnSelect)
						viewItems.children[index].color = style.backgroundSelectedColor;

					if (view.width == '100%')
						cursor.left = ((100 / items.length) * index) + '%';
					else
						cursor.left = (view.width / items.length) * index;
				}
			};

			for (var i = 0; i < items.length; i++) {
				var lbl = Ti.UI.createLabel({
					top : 6,
					width : lblWidth,
					color : style.color,
					font : style.font,
					textAlign : 'center',
					text : items[i],
					index : i,
					horizontalWrap : false,
					height : style.height - 12,
					wordWrap : true
				});
				lbl.addEventListener('click', function() {
					view.index = this.index;
					view.setPosition(this.index);
					view.fireEvent('toolkit_SelectorSelected', {
						text : this.text,
						index : this.index
					});
				});
				viewItems.add(lbl);
			}
			if (items.length > 0) {
				view.setPosition(style.index);
			}

			view.getIndex = function() {
				return view.index;
			};
			view.setIndex = function(index) {
				view.index = index;
				view.setPosition(index);
			};

			return view;
		}
	},
	TabbedBar : {
		GetOptions : function() {
			return {
				height : 30,
				width : '100%',
				left : 0,
				top : 0,
				backgroundColor : 'black',
				color : 'silver',
				backgroundSelectedColor : 'white',
				colorSelectedColor : 'black',
				index : 0,
				font : {
					fontSize : 10
				},
				changeColorTextOnSelect : true
			};
		},
		Create : function(items, style) {
			var defaults = exports.Controls.TabbedBar.GetOptions();
			style = exports.Utility.extend(style, defaults);

			var lblWidth = parseInt(100 / items.length);

			var view = Ti.UI.createView({
				height : style.height,
				width : style.width,
				left : style.left,
				top : style.top,
				backgroundColor : style.backgroundColor,
				index : style.index,
				layout : 'horizontal'
			});

			for (var i = 0; i < items.length; i++) {
				var lbl = Ti.UI.createLabel({
					top : 0,
					width : (i != (items.length - 1) ? lblWidth : (100 - (lblWidth * (items.length - 1)))) + '%',
					height : style.height,
					color : style.color,
					backgroundColor : style.backgroundColor,
					font : style.font,
					textAlign : 'center',
					text : items[i],
					index : i
				});
				lbl.addEventListener('click', function() {
					view.index = this.index;
					for (var i = 0; i < view.children.length; i++) {
						view.children[i].color = style.color;
						view.children[i].backgroundColor = style.backgroundColor;
					}
					this.color = style.colorSelectedColor;
					this.backgroundColor = style.backgroundSelectedColor;
					view.fireEvent('toolkit_TabbedBarSelected', {
						text : this.text,
						index : this.index
					});
				});
				view.add(lbl);
			}
			if (items.length > 0) {
				view.children[style.index].color = style.colorSelectedColor;
				view.children[style.index].backgroundColor = style.backgroundSelectedColor;
			}

			view.getIndex = function() {
				return view.index;
			};
			view.setIndex = function(index) {
				view.index = index;
				for (var i = 0; i < view.children.length; i++) {
					view.children[i].color = style.color;
					view.children[i].backgroundColor = style.backgroundColor;
				}
				view.children[index].color = style.colorSelectedColor;
				view.children[index].backgroundColor = style.backgroundSelectedColor;
			};

			return view;
		}
	},
	Picker : {
		RemoveAllRows : function(picker) {
			if (picker.columns[0]) {
				var col = picker.columns[0];
				var len = col.rowCount;
				for (var x = len - 1; x >= 0; x--) {
					var row = col.rows[x];
					col.removeRow(row);
				}
			}
		}
	},
	SetHasFocus : function(ctrl) {
		ctrl.addEventListener('focus', function() {
			this.hasFocus = true;
		});

		ctrl.addEventListener('blur', function() {
			this.hasFocus = false;
		});
	},
	SetIndicator : function(ctrl) {
		ctrl.preventDefaultImage = true;
		ctrl.ActivityIndicator = Ti.UI.createActivityIndicator({

		});
		ctrl.add(ctrl.ActivityIndicator);
		ctrl.ActivityIndicator.show();

		ctrl.RemoteImage = function(image, defaultImage) {
			if (image == null || image == '') {
				ctrl.ActivityIndicator.hide();
				if (defaultImage != null && defaultImage != '')
					ctrl.image = defaultImage;
			} else {
				ctrl.ActivityIndicator.show();
				ctrl.image = image;
			}
		};

		ctrl.addEventListener('load', function() {
			ctrl.ActivityIndicator.hide();
		});
	}
};
//INDICATOR
Indicator = {
	// Activity Indicator
	indWin : null,
	indAct : null,
	indicatorShowing : false,
	hideIndicator : function() {
		if (this.indicatorShowing == true) {
			if (this.indAct) {
				this.indAct.hide();
			}
			if (this.indWin != null) {
				this.indWin.close();
			}
			this.indicatorShowing = false;
		}
	},
	showIndicator : function(title) {
		this.indicatorShowing = true;
		// window container
		if (this.indWin == null) {
			this.indWin = Titanium.UI.createWindow({
				backgroundColor : 'transparent',
				height : 480,
				width : 320,
				zIndex : 0
			});

			// black view
			var indView = Titanium.UI.createView({
				height : 80,
				width : 200,
				backgroundColor : '#000',
				borderRadius : 10,
				opacity : 0.7,
				borderColor : 'silver',
				borderWidth : 2
			});

			this.indAct = Titanium.UI.createActivityIndicator({
				top : 15,
				height : 30,
				width : 30
			});
			if (exports.Environment.isiOS)
				this.indAct.style = Titanium.UI.iPhone.ActivityIndicatorStyle.PLAIN;

			indView.add(this.indAct);

			var indLbl = Ti.UI.createLabel({
				top : 50,
				font : {
					fontFamily : 'Helvetica Neue',
					fontSize : 15,
					fontWeight : 'bold'
				},
				color : '#fff',
				text : title
			});
			indView.add(indLbl);

			this.indWin.add(indView);
		}

		this.indWin.zIndex = 0;

		this.indAct.show();
		this.indWin.open();
	}
};
//TABLE
exports.Table = {
	HeaderPull : {
		GetOptions : function() {
			return {
				backgroundColor : 'darkgray',
				borderBackgroundColor : 'gray',
				borderHeight : 1,
				color : '#fff',
				textDefault : "Tira giu' per aggiornare...",
				textRefresh : "Rilascia per aggiornare...",
				textOnRefresh : "Aggiornamento..."
			};
		},
		Create : function(options, table, onRefresh) {
			var defaults = exports.Table.HeaderPull.GetOptions();
			options = exports.Utility.extend(options, defaults);

			var header = Ti.UI.createView({
				backgroundColor : options.backgroundColor,
				width : 320,
				height : 60
			});
			exports.Drawing.BorderBottom(header, options.borderBackgroundColor, options.borderHeight);

			var arrow = Ti.UI.createView({
				backgroundImage : "/images/toolkit/arrow_white.png",
				width : 23,
				height : 60,
				bottom : 10,
				left : 20
			});
			var status = Ti.UI.createLabel({
				left : 60,
				width : 200,
				bottom : 22,
				height : "auto",
				color : options.color,
				textAlign : "left",
				font : {
					fontSize : 13,
					fontWeight : "bold"
				},
				shadowColor : "#333",
				shadowOffset : {
					x : 0,
					y : 1
				}
			});
			var actInd = Titanium.UI.createActivityIndicator({
				left : 20,
				bottom : 13,
				width : 30,
				height : 30
			});

			header.add(arrow);
			header.add(status);
			header.add(actInd);
			table.headerPullView = header;

			var pulling = false;
			var reloading = false;

			table.addEventListener('scroll', function(e) {
				if (e.contentOffset!=undefined) {
					var offset = e.contentOffset.y;
					if (offset <= -65.0 && !pulling && !reloading) {
						var t = Ti.UI.create2DMatrix();
						t = t.rotate(-180);
						pulling = true;
						arrow.animate({
							transform : t,
							duration : 180
						});
						status.text = options.textRefresh;
					} else if (pulling && (offset > -65.0 && offset < 0) && !reloading) {
						pulling = false;
						var t = Ti.UI.create2DMatrix();
						arrow.animate({
							transform : t,
							duration : 180
						});
						status.text = options.textDefault;
					}
				}
			});
			var event1 = 'dragEnd';
			if (Ti.version >= '3.0.0') {
				event1 = 'dragend';
			}
			table.addEventListener(event1, function(e) {
				if (pulling && !reloading) {
					reloading = true;
					pulling = false;
					arrow.hide();
					actInd.show();
					status.text = options.textOnRefresh;
					table.setContentInsets({
						top : 60
					}, {
						animated : true
					});
					arrow.transform = Ti.UI.create2DMatrix();

					onRefresh();
				}
			});

			return function() {
				table.setContentInsets({
					top : 0
				}, {
					animated : true
				});
				reloading = false;
				status.text = options.textDefault;
				actInd.hide();
				arrow.show();
			};
		}
	},
	Separator : {
		Create : function(tvr) {
			exports.Drawing.BorderTop(tvr, '#333333');
			exports.Drawing.BorderBottom(tvr, '#0f0f0f');
		}
	}
};
//DRAWING
exports.Drawing = {
	PixelsToDPUnits : function(ThePixels) {
		return (ThePixels / (Titanium.Platform.displayCaps.dpi / 160));
	},
	DPUnitsToPixels : function(TheDPUnits) {
		return (TheDPUnits * (Titanium.Platform.displayCaps.dpi / 160));
	},
	Color : {
		Random : function() {
			return '#' + (0x1000000 + (Math.random()) * 0xffffff).toString(16).substr(1, 6);
		}
	},
	Separator : function() {
		var self = Ti.UI.createView({
			height : 2
		});
		exports.Drawing.BorderTop(self, '#0f0f0f');
		exports.Drawing.BorderTop(self, '#333333');

		return self;
	},
	BorderTop : function(container, color, borderWidth) {
		return exports.Drawing.Border(container, color, borderWidth, 'top');
	},
	BorderRight : function(container, color, borderWidth) {
		return exports.Drawing.Border(container, color, borderWidth, 'right');
	},
	BorderBottom : function(container, color, borderWidth) {
		return exports.Drawing.Border(container, color, borderWidth, 'bottom');
	},
	BorderLeft : function(container, color, borderWidth) {
		return exports.Drawing.Border(container, color, borderWidth, 'left');
	},
	BorderAll : function(container, color, borderWidth) {
		return exports.Drawing.BorderTop(container, color, borderWidth) && exports.Drawing.BorderRight(container, color, borderWidth) && exports.Drawing.BorderBottom(container, color, borderWidth) && exports.Drawing.BorderLeft(container, color, borderWidth);
	},
	Border : function(container, color, borderWidth, position) {
		color = color || 'black';
		borderWidth = borderWidth || 1;
		position = position || 'top';

		var border = Ti.UI.createView({
			backgroundColor : color
		});

		switch (position) {
			case 'top':
				border.top = 0;
				border.left = 0;
				border.right = 0;
				border.height = borderWidth;
				break;
			case 'right':
				border.top = 0;
				border.bottom = 0;
				border.right = 0;
				border.width = borderWidth;
				break;
			case 'bottom':
				border.bottom = 0;
				border.left = 0;
				border.right = 0;
				border.height = borderWidth;
				break;
			case 'left':
				border.top = 0;
				border.left = 0;
				border.width = borderWidth;
				border.bottom = 0;
				break;
		}

		container.add(border);
		return true;
	}
};
//NET
exports.NET = {
	GetOptions : function() {
		return {
			headers : [],
			timeout : 60000,
			validatesSecureCertificate : false,
			responseType : 'TEXT',
			method : 'GET'
		};
	},
	SetHeader : function(xhr, headers) {
		for (var i = 0; i < headers.length; i++) {
			xhr.setRequestHeader(headers[i].name, headers[i].value);
		}
		return xhr;
	},
	SetHeaderItem : function(headers, itemName, itemValue) {
		headers.push({
			name : itemName,
			value : itemValue
		});
		return headers;
	},
	SetHeaderBasicAuthorization : function(headers, username, password) {
		var authostring = "Basic " + Ti.Utils.base64encode(username + ":" + password);
		return exports.NET.SetHeaderItem(headers, "Authorization", authostring);
	},
	Call : function(uri, parameters, options, onSuccess, onError) {
		var defaults = exports.NET.GetOptions();
		options = exports.Utility.extend(options, defaults);

		if (!Ti.Network.online) {
			//Recupero currentObject con chiave <url> + <username>
			//...se lo trovo...ritorno onSuccess(currentObject)
			var currentObject = Ti.App.Properties.getString(uri, null);
			if (currentObject != null) {
				onSuccess(JSON.parse(currentObject));
			} else {
				if (onError != null) {
					var e = {
						error : "Errore cache, riprovare!"
					};
					onError(e);
				}
			}
			return;
		}

		var xhr = Ti.Network.createHTTPClient();
		xhr.setValidatesSecureCertificate(options.validatesSecureCertificate);
		xhr.setTimeout(options.timeout);
		xhr.onerror = function(e) {
			if (onError != null) {
				onError(e, xhr.status);
			}
		};
		xhr.onload = function() {
			if (onSuccess != null) {
				try {
					switch (options.responseType) {
						case 'TEXT':
							var response = xhr.responseText;
							if (response != null && response != 'null') {
								var currentObject = JSON.parse(response);
								if (currentObject == null) {
									var e = {
										error : "Errore JSON parse, riprovare!"
									};
									onError(e);
								} else {
									onSuccess(currentObject);
								}
							} else {
								onSuccess({});
							}
							break;
						case 'XML':
							onSuccess(xhr.responseXML);
							break;
						case 'DATA':
							onSuccess(xhr.responseData);
							break;
					}

				} catch(err) {
					if (onError != null) {
						onError({
							error : err
						}, xhr.status);
					}
				}
			}
		};

		xhr.open(options.method, uri);
		xhr = exports.NET.SetHeader(xhr, options.headers);
		xhr.send(parameters);
	},
	DownloadFile : function(uri, options, filepath, onError, onSuccess, onProgress) {
		var defaults = exports.NET.getOptions();
		options = exports.Utility.extend(options, defaults);

		if (Titanium.Network.online) {
			var xhr = Titanium.Network.createHTTPClient();
			xhr.setValidatesSecureCertificate(options.validatesSecureCertificate);
			xhr.setTimeout(options.timeout);

			xhr.onload = function() {
				if (xhr.status == 200) {
					var f = Titanium.Filesystem.getFile(filepath);
					f.write(this.responseData);
					f = null;
					onSuccess();
				} else {
					onError({
						error : 'Non e\' possibile scaricare il file.'
					}, xhr.status);
				}
			};
			xhr.ondatastream = function(e) {
				if (onProgress)
					onProgress(e.progress);
			};
			xhr.error = function(e) {
				onError(e);
			};

			xhr.open('GET', uri);
			xhr = setHeader(xhr, options.headers);
			xhr.send();
		} else {
			onError({
				error : 'Non �� presente una connessione attiva.'
			});
		}
	},
	UploadFile : function(uri, filename, filepath, options, onError, onSuccess, onProgress) {
		var defaults = exports.NET.getOptions();
		options = exports.Utility.extend(options, defaults);

		var upload = function(file) {
			var xhr = Ti.Network.createHTTPClient();
			xhr.setValidatesSecureCertificate(options.validatesSecureCertificate);
			xhr.setTimeout(options.timeout);

			xhr.onload = function() {
				onSuccess();
			};
			xhr.onsendstream = function(e) {
				Ti.API.info('uploadFile progress: ' + e.progress * 100);
				if (onProgress)
					onProgress(e.progress);
			};
			xhr.onerror = function(e) {
				onError(e);
			};

			xhr.open('POST', uri);
			xhr.setRequestHeader('enctype', 'multipart/form-data');
			xhr.setRequestHeader('filename', filename);
			xhr = setHeader(xhr, options.headers);
			xhr.send(data);
		};

		var f = Titanium.Filesystem.getFile(filepath);
		if (f.exists()) {
			var file = f.read();
			upload(file);
		} else {
			onError({
				error : 'Il file non esiste!'
			});
		}
		f = null;
	}
};
//DATABASE
exports.Database = {
	Open : function(name) {
		return Ti.Database.open(name);
	},
	Close : function(db) {
		return db.close();
	},
	Execute : function(db, sql, parameters, onSuccess, onError) {
		try {
			var rs;

			if (parameters != null)
				rs = db.execute(sql, parameters);
			else
				rs = db.execute(sql);

			onSuccess(rs);
		} catch(err) {
			onError({
				error : err
			});
		}
	},
	Init : function(db, tablename, columns, onSuccess, onError) {
		try {
			if (tablename == '')
				throw {
					error : 'Non e\' stato specificato il nome della tabella'
				};
			if (columns.length == 0)
				throw {
					error : 'Non sono state specificate le colonne'
				};

			var sql = 'CREATE TABLE IF NOT EXISTS ' + tablename + ' ( ';

			for (var i = 0; i < columns.length; i++) {
				sql += columns[i].name + ' ' + columns[i].type + ' ' + (columns[i].request ? 'NOT NULL' : '') + ' ' + (columns[i].isPrimary ? 'PRIMARY KEY' : '') + ',';
			}
			sql = sql.substring(0, sql.length - 1);
			sql += ' )';

			exports.Database.Execute(db, sql, null, onSuccess, onError);
		} catch(err) {
			onError({
				error : err
			});
		}
	},
	Select : function(db, tablename, condition, parameters, onSuccess, onError) {
		try {
			if (tablename == '')
				throw {
					error : 'Non e\' stato specificato il nome della tabella'
				};

			var sql = 'SELECT * FROM ' + tablename;
			if (condition != null && condition != '')
				sql += ' WHERE ' + condition;

			exports.Database.Execute(db, sql, parameters, onSuccess, onError);
		} catch(err) {
			onError({
				error : err
			});
		}
	},
	Insert : function(db, tablename, columns, parameters, onSuccess, onError) {
		try {
			if (tablename == '')
				throw {
					error : 'Non e\' stato specificato il nome della tabella'
				};
			if (columns.length == 0)
				throw {
					error : 'Non sono state specificate le colonne'
				};

			var sql = 'INSERT INTO ' + tablename + ' (';
			for (var i = 0; i < columns.length; i++) {
				sql += columns[i].name + ',';
			}
			sql = sql.substring(0, sql.length - 1) + ') VALUES (';
			for (var i = 0; i < columns.length; i++) {
				sql += '?,';
			}
			sql = sql.substring(0, sql.length - 1) + ')';

			exports.Database.Execute(db, sql, parameters, onSuccess, onError);
		} catch(err) {
			onError({
				error : err
			});
		}
	},
	Update : function(db, tablename, columns, condition, parameters, onSuccess, onError) {
		try {
			if (tablename == '')
				throw {
					error : 'Non e\' stato specificato il nome della tabella'
				};
			if (columns.length == 0)
				throw {
					error : 'Non sono state specificate le colonne'
				};

			var sql = 'UPDATE ' + tablename + ' SET ';
			for (var i = 0; i < columns.length; i++) {
				sql += columns[i].name + '=?,';
			}
			sql = sql.substring(0, sql.length - 1);
			if (condition != null && condition != '')
				sql += ' WHERE ' + condition;

			exports.Database.Execute(db, sql, parameters, onSuccess, onError);
		} catch(err) {
			onError({
				error : err
			});
		}
	},
	Delete : function(db, tablename, condition, parameters, onSuccess, onError) {
		try {
			var sql = 'DELETE FROM ' + tablename;
			if (condition != null && condition != '')
				sql += ' WHERE ' + condition;

			db.execute(sql, parameters);
			onSuccess();
			//exports.Database.Execute(db, sql, parameters, onSuccess, onError);
		} catch(err) {
			onError({
				error : err
			});
		}
	}
};
//GOOGLE
exports.Google = {
	Chart : {
		getHtml : function(type, data, options) {
			var html = '<html xmlns="http://www.w3.org/1999/xhtml">';
			html += '<head>';
			html += '	<meta http-equiv="content-type" content="text/html; charset=utf-8"/>';
			html += '	<title></title>';
			html += '	<script type="text/javascript" src="http://www.google.com/jsapi"></script>';
			html += '	<script type="text/javascript">';
			html += '	 google.load("visualization", "1", {packages:["corechart"]});';
			html += '	 google.setOnLoadCallback(drawChart);';
			html += '	 function drawChart() {';
			html += '	  var data = new google.visualization.DataTable(' + data + ');';
			html += '	  var options = ' + options + ';';
			html += '	  var chart = new google.visualization.' + type + '(document.getElementById("chart_div"));';
			html += '	  chart.draw(data, options);';
			html += '	 }';
			html += '	</script>';
			html += '</head>';
			html += '<body style="font-family: Arial;border: 0 none;padding:0px;margin:0px">';
			html += '	<div id="chart_div"></div>';
			html += '</body>';
			html += '</html>';

			return html;
		}
	}
};
//RSS
exports.Xml = {
	getText : function(item, key) {
		return item.getElementsByTagName(key).item(0).text;
	},
	getAttribute : function(item, key, attribute) {
		var node = item.getElementsByTagName(key);
		if (node && node.length > 0) {
			return node.item(0).getAttribute(attribute);
		} else
			return null;
	}
};
//HTML
exports.Html = {
	Encoder : {

		// When encoding do we convert characters into html or numerical entities
		EncodeType : "entity", // entity OR numerical

		isEmpty : function(val) {
			if (val) {
				return ((val === null) || val.length == 0 || /^\s+$/.test(val));
			} else {
				return true;
			}
		},

		// arrays for conversion from HTML Entities to Numerical values
		arr1 : ['&nbsp;', '&iexcl;', '&cent;', '&pound;', '&curren;', '&yen;', '&brvbar;', '&sect;', '&uml;', '&copy;', '&ordf;', '&laquo;', '&not;', '&shy;', '&reg;', '&macr;', '&deg;', '&plusmn;', '&sup2;', '&sup3;', '&acute;', '&micro;', '&para;', '&middot;', '&cedil;', '&sup1;', '&ordm;', '&raquo;', '&frac14;', '&frac12;', '&frac34;', '&iquest;', '&Agrave;', '&Aacute;', '&Acirc;', '&Atilde;', '&Auml;', '&Aring;', '&AElig;', '&Ccedil;', '&Egrave;', '&Eacute;', '&Ecirc;', '&Euml;', '&Igrave;', '&Iacute;', '&Icirc;', '&Iuml;', '&ETH;', '&Ntilde;', '&Ograve;', '&Oacute;', '&Ocirc;', '&Otilde;', '&Ouml;', '&times;', '&Oslash;', '&Ugrave;', '&Uacute;', '&Ucirc;', '&Uuml;', '&Yacute;', '&THORN;', '&szlig;', '&agrave;', '&aacute;', '&acirc;', '&atilde;', '&auml;', '&aring;', '&aelig;', '&ccedil;', '&egrave;', '&eacute;', '&ecirc;', '&euml;', '&igrave;', '&iacute;', '&icirc;', '&iuml;', '&eth;', '&ntilde;', '&ograve;', '&oacute;', '&ocirc;', '&otilde;', '&ouml;', '&divide;', '&oslash;', '&ugrave;', '&uacute;', '&ucirc;', '&uuml;', '&yacute;', '&thorn;', '&yuml;', '&quot;', '&amp;', '&lt;', '&gt;', '&OElig;', '&oelig;', '&Scaron;', '&scaron;', '&Yuml;', '&circ;', '&tilde;', '&ensp;', '&emsp;', '&thinsp;', '&zwnj;', '&zwj;', '&lrm;', '&rlm;', '&ndash;', '&mdash;', '&lsquo;', '&rsquo;', '&sbquo;', '&ldquo;', '&rdquo;', '&bdquo;', '&dagger;', '&Dagger;', '&permil;', '&lsaquo;', '&rsaquo;', '&euro;', '&fnof;', '&Alpha;', '&Beta;', '&Gamma;', '&Delta;', '&Epsilon;', '&Zeta;', '&Eta;', '&Theta;', '&Iota;', '&Kappa;', '&Lambda;', '&Mu;', '&Nu;', '&Xi;', '&Omicron;', '&Pi;', '&Rho;', '&Sigma;', '&Tau;', '&Upsilon;', '&Phi;', '&Chi;', '&Psi;', '&Omega;', '&alpha;', '&beta;', '&gamma;', '&delta;', '&epsilon;', '&zeta;', '&eta;', '&theta;', '&iota;', '&kappa;', '&lambda;', '&mu;', '&nu;', '&xi;', '&omicron;', '&pi;', '&rho;', '&sigmaf;', '&sigma;', '&tau;', '&upsilon;', '&phi;', '&chi;', '&psi;', '&omega;', '&thetasym;', '&upsih;', '&piv;', '&bull;', '&hellip;', '&prime;', '&Prime;', '&oline;', '&frasl;', '&weierp;', '&image;', '&real;', '&trade;', '&alefsym;', '&larr;', '&uarr;', '&rarr;', '&darr;', '&harr;', '&crarr;', '&lArr;', '&uArr;', '&rArr;', '&dArr;', '&hArr;', '&forall;', '&part;', '&exist;', '&empty;', '&nabla;', '&isin;', '&notin;', '&ni;', '&prod;', '&sum;', '&minus;', '&lowast;', '&radic;', '&prop;', '&infin;', '&ang;', '&and;', '&or;', '&cap;', '&cup;', '&int;', '&there4;', '&sim;', '&cong;', '&asymp;', '&ne;', '&equiv;', '&le;', '&ge;', '&sub;', '&sup;', '&nsub;', '&sube;', '&supe;', '&oplus;', '&otimes;', '&perp;', '&sdot;', '&lceil;', '&rceil;', '&lfloor;', '&rfloor;', '&lang;', '&rang;', '&loz;', '&spades;', '&clubs;', '&hearts;', '&diams;'],
		arr2 : ['&#160;', '&#161;', '&#162;', '&#163;', '&#164;', '&#165;', '&#166;', '&#167;', '&#168;', '&#169;', '&#170;', '&#171;', '&#172;', '&#173;', '&#174;', '&#175;', '&#176;', '&#177;', '&#178;', '&#179;', '&#180;', '&#181;', '&#182;', '&#183;', '&#184;', '&#185;', '&#186;', '&#187;', '&#188;', '&#189;', '&#190;', '&#191;', '&#192;', '&#193;', '&#194;', '&#195;', '&#196;', '&#197;', '&#198;', '&#199;', '&#200;', '&#201;', '&#202;', '&#203;', '&#204;', '&#205;', '&#206;', '&#207;', '&#208;', '&#209;', '&#210;', '&#211;', '&#212;', '&#213;', '&#214;', '&#215;', '&#216;', '&#217;', '&#218;', '&#219;', '&#220;', '&#221;', '&#222;', '&#223;', '&#224;', '&#225;', '&#226;', '&#227;', '&#228;', '&#229;', '&#230;', '&#231;', '&#232;', '&#233;', '&#234;', '&#235;', '&#236;', '&#237;', '&#238;', '&#239;', '&#240;', '&#241;', '&#242;', '&#243;', '&#244;', '&#245;', '&#246;', '&#247;', '&#248;', '&#249;', '&#250;', '&#251;', '&#252;', '&#253;', '&#254;', '&#255;', '&#34;', '&#38;', '&#60;', '&#62;', '&#338;', '&#339;', '&#352;', '&#353;', '&#376;', '&#710;', '&#732;', '&#8194;', '&#8195;', '&#8201;', '&#8204;', '&#8205;', '&#8206;', '&#8207;', '&#8211;', '&#8212;', '&#8216;', '&#8217;', '&#8218;', '&#8220;', '&#8221;', '&#8222;', '&#8224;', '&#8225;', '&#8240;', '&#8249;', '&#8250;', '&#8364;', '&#402;', '&#913;', '&#914;', '&#915;', '&#916;', '&#917;', '&#918;', '&#919;', '&#920;', '&#921;', '&#922;', '&#923;', '&#924;', '&#925;', '&#926;', '&#927;', '&#928;', '&#929;', '&#931;', '&#932;', '&#933;', '&#934;', '&#935;', '&#936;', '&#937;', '&#945;', '&#946;', '&#947;', '&#948;', '&#949;', '&#950;', '&#951;', '&#952;', '&#953;', '&#954;', '&#955;', '&#956;', '&#957;', '&#958;', '&#959;', '&#960;', '&#961;', '&#962;', '&#963;', '&#964;', '&#965;', '&#966;', '&#967;', '&#968;', '&#969;', '&#977;', '&#978;', '&#982;', '&#8226;', '&#8230;', '&#8242;', '&#8243;', '&#8254;', '&#8260;', '&#8472;', '&#8465;', '&#8476;', '&#8482;', '&#8501;', '&#8592;', '&#8593;', '&#8594;', '&#8595;', '&#8596;', '&#8629;', '&#8656;', '&#8657;', '&#8658;', '&#8659;', '&#8660;', '&#8704;', '&#8706;', '&#8707;', '&#8709;', '&#8711;', '&#8712;', '&#8713;', '&#8715;', '&#8719;', '&#8721;', '&#8722;', '&#8727;', '&#8730;', '&#8733;', '&#8734;', '&#8736;', '&#8743;', '&#8744;', '&#8745;', '&#8746;', '&#8747;', '&#8756;', '&#8764;', '&#8773;', '&#8776;', '&#8800;', '&#8801;', '&#8804;', '&#8805;', '&#8834;', '&#8835;', '&#8836;', '&#8838;', '&#8839;', '&#8853;', '&#8855;', '&#8869;', '&#8901;', '&#8968;', '&#8969;', '&#8970;', '&#8971;', '&#9001;', '&#9002;', '&#9674;', '&#9824;', '&#9827;', '&#9829;', '&#9830;'],

		// Convert HTML entities into numerical entities
		HTML2Numerical : function(s) {
			return this.swapArrayVals(s, this.arr1, this.arr2);
		},

		// Convert Numerical entities into HTML entities
		NumericalToHTML : function(s) {
			return this.swapArrayVals(s, this.arr2, this.arr1);
		},

		// Numerically encodes all unicode characters
		numEncode : function(s) {
			if (this.isEmpty(s))
				return "";

			var a = [], l = s.length;

			for (var i = 0; i < l; i++) {
				var c = s.charAt(i);
				if (c < " " || c > "~") {
					a.push("&#");
					a.push(c.charCodeAt());
					//numeric value of code point
					a.push(";");
				} else {
					a.push(c);
				}
			}

			return a.join("");
		},

		// HTML Decode numerical and HTML entities back to original values
		htmlDecode : function(s) {

			var c, m, d = s;

			if (this.isEmpty(d))
				return "";

			// convert HTML entites back to numerical entites first
			d = this.HTML2Numerical(d);

			// look for numerical entities &#34;
			arr = d.match(/&#[0-9]{1,5};/g);

			// if no matches found in string then skip
			if (arr != null) {
				for (var x = 0; x < arr.length; x++) {
					m = arr[x];
					c = m.substring(2, m.length - 1);
					//get numeric part which is refernce to unicode character
					// if its a valid number we can decode
					if (c >= -32768 && c <= 65535) {
						// decode every single match within string
						d = d.replace(m, String.fromCharCode(c));
					} else {
						d = d.replace(m, "");
						//invalid so replace with nada
					}
				}
			}

			return d;
		},

		// encode an input string into either numerical or HTML entities
		htmlEncode : function(s, dbl) {

			if (this.isEmpty(s))
				return "";

			// do we allow double encoding? E.g will &amp; be turned into &amp;amp;
			dbl = dbl || false;
			//default to prevent double encoding

			// if allowing double encoding we do ampersands first
			if (dbl) {
				if (this.EncodeType == "numerical") {
					s = s.replace(/&/g, "&#38;");
				} else {
					s = s.replace(/&/g, "&amp;");
				}
			}

			// convert the xss chars to numerical entities ' " < >
			s = this.XSSEncode(s, false);

			if (this.EncodeType == "numerical" || !dbl) {
				// Now call function that will convert any HTML entities to numerical codes
				s = this.HTML2Numerical(s);
			}

			// Now encode all chars above 127 e.g unicode
			s = this.numEncode(s);

			// now we know anything that needs to be encoded has been converted to numerical entities we
			// can encode any ampersands & that are not part of encoded entities
			// to handle the fact that I need to do a negative check and handle multiple ampersands &&&
			// I am going to use a placeholder

			// if we don't want double encoded entities we ignore the & in existing entities
			if (!dbl) {
				s = s.replace(/&#/g, "##AMPHASH##");

				if (this.EncodeType == "numerical") {
					s = s.replace(/&/g, "&#38;");
				} else {
					s = s.replace(/&/g, "&amp;");
				}

				s = s.replace(/##AMPHASH##/g, "&#");
			}

			// replace any malformed entities
			s = s.replace(/&#\d*([^\d;]|$)/g, "$1");

			if (!dbl) {
				// safety check to correct any double encoded &amp;
				s = this.correctEncoding(s);
			}

			// now do we need to convert our numerical encoded string into entities
			if (this.EncodeType == "entity") {
				s = this.NumericalToHTML(s);
			}

			return s;
		},

		// Encodes the basic 4 characters used to malform HTML in XSS hacks
		XSSEncode : function(s, en) {
			if (!this.isEmpty(s)) {
				en = en || true;
				// do we convert to numerical or html entity?
				if (en) {
					s = s.replace(/\'/g, "&#39;");
					//no HTML equivalent as &apos is not cross browser supported
					s = s.replace(/\"/g, "&quot;");
					s = s.replace(/</g, "&lt;");
					s = s.replace(/>/g, "&gt;");
				} else {
					s = s.replace(/\'/g, "&#39;");
					//no HTML equivalent as &apos is not cross browser supported
					s = s.replace(/\"/g, "&#34;");
					s = s.replace(/</g, "&#60;");
					s = s.replace(/>/g, "&#62;");
				}
				return s;
			} else {
				return "";
			}
		},

		// returns true if a string contains html or numerical encoded entities
		hasEncoded : function(s) {
			if (/&#[0-9]{1,5};/g.test(s)) {
				return true;
			} else if (/&[A-Z]{2,6};/gi.test(s)) {
				return true;
			} else {
				return false;
			}
		},

		// will remove any unicode characters
		stripUnicode : function(s) {
			return s.replace(/[^\x20-\x7E]/g, "");

		},

		// corrects any double encoded &amp; entities e.g &amp;amp;
		correctEncoding : function(s) {
			return s.replace(/(&amp;)(amp;)+/, "$1");
		},

		// Function to loop through an array swaping each item with the value from another array e.g swap HTML entities with Numericals
		swapArrayVals : function(s, arr1, arr2) {
			if (this.isEmpty(s))
				return "";
			var re;
			if (arr1 && arr2) {
				//ShowDebug("in swapArrayVals arr1.length = " + arr1.length + " arr2.length = " + arr2.length)
				// array lengths must match
				if (arr1.length == arr2.length) {
					for (var x = 0, i = arr1.length; x < i; x++) {
						re = new RegExp(arr1[x], 'g');
						s = s.replace(re, arr2[x]);
						//swap arr1 item with matching item from arr2
					}
				}
			}
			return s;
		},

		inArray : function(item, arr) {
			for (var i = 0, x = arr.length; i < x; i++) {
				if (arr[i] === item) {
					return i;
				}
			}
			return -1;
		}
	}
};
//Geolocation
exports.Geolocation = {
	IsEnabled : function(onSuccess) {
		if (exports.Environment.isApple) {
			Titanium.Geolocation.getCurrentPosition(function(e) {
				if (e.coords)
					onSuccess(true);
				else
					onSuccess(false);
			});
		}
		if (exports.Environment.isAndroid) {
			//if (Titanium.Geolocation.isLocationProviderEnabled(Titanium.Geolocation.PROVIDER_GPS, Titanium.Geolocation.ACCURACY_BEST) == false)
			//	onSuccess(false);
			//else
			onSuccess(true);
		}
		return false;
	},
	OpenMessage : function() {
		var alertDlg = Titanium.UI.createAlertDialog({
			title : Titanium.App.name,
			message : 'Abilitare GPS da impostazioni.',
			buttonNames : ['Annulla']
		});
		alertDlg.show();
	}
};
//Platform
exports.Platform = {
	OpenUrl : function(url) {
		Ti.Platform.openURL(url);
	},
	CallNumber : function(phonenumber) {
		Titanium.Platform.openURL('tel:' + phonenumber);
	}
};
//GLOBAL EVENTS
Titanium.App.addEventListener('toolkit_IndicatorShow', function(e) {
	if (e.title == null) {
		e.title = 'Attendere prego... ';
	}
	Indicator.showIndicator(e.title);
});
Titanium.App.addEventListener('toolkit_IndicatorHide', function(e) {
	Indicator.hideIndicator();
});
