var toolkit = require('/lib/toolkit');
var rules = require('/lib/rules');
var rss = require('/lib/rss');

function getRow(item) {
	var self = Ti.UI.createTableViewRow({
		className : 'NewCategoryRowView',
		hasChild : true,
		backgroundColor : rules.settings.Styles.TableRow.backgroundColor,
		selectedBackgroundColor : rules.settings.Styles.TableRow.selectedBackgroundColor,
		height : 100,
		tag : item
	});

	if (toolkit.Environment.isAndroid && toolkit.Environment.isTablet) {
		self.height=(Ti.Platform.displayCaps.platformWidth/3);
	}

	//Immagine
	var img = Ti.UI.createImageView({
		top : 5,
		bottom : 5,
		left : 5,
		width : self.height - (self.height*0.1),
		borderColor : 'white',
		borderWidth : 1
	});
	self.add(img);

	var lblPubDate = Ti.UI.createLabel({
		backgroundColor : rules.settings.Styles.Window.backgroundColor,
		bottom : 6,
		left : 6,
		width : img.width-2,
		height : (self.height*0.2),
		font : {
			fontSize : (self.height*0.08)
		},
		text : item.pubdate,
		textAlign : 'center',
		color : 'white'
	});
	self.add(lblPubDate);

	toolkit.Controls.SetIndicator(img);
	img.RemoteImage(item.imageurl, '/images/logo/Default.png');

	//Title
	var lblTitle = Ti.UI.createLabel({
		top : 5,
		right : 5,
		left : self.height + 5,
		height : (self.height*0.16),
		text : item.title,
		font : {
			fontSize : (self.height*0.12)
		},
		color : 'white'
	});
	self.add(lblTitle);

	//Description
	var lblDescription = Ti.UI.createLabel({
		top : (self.height*0.20) + 5,
		right : 5,
		bottom : 5,
		left : self.height + 5,
		text : item.description,
		color : 'white',
		font : {
			fontSize : (self.height*0.1)
		},
		verticalAlign : Ti.UI.TEXT_VERTICAL_ALIGNMENT_TOP
	});
	self.add(lblDescription);

	return self;
}

function getView(rsslink) {
	var self = Ti.UI.createView();

	var table = Ti.UI.createTableView({
		data : [],
		separatorColor : rules.settings.Styles.TableRow.separatorColor,
		backgroundColor : 'transparent'
	});
	self.add(table);
	table.addEventListener('click', function(e) {
		table.selectRow(e.index);
		self.fireEvent('selectedItem', e.rowData.tag);
	});

	//Fill table
	self.fill = function(force) {
		if (table.data.length == 0 || force) {
			table.data = [];

			rss.Get(rsslink, {
				success : function(data) {
					for (var i = 0; i < data.length; i++) {
						table.appendRow(getRow(data[i]));
					}

					if (toolkit.Environment.isAndroid)
						HeaderPullComplete();
				},
				error : function() {
					toolkit.Dialog.Error(L('notconnection'));
					if (toolkit.Environment.isAndroid)
						HeaderPullComplete();
				}
			});
		}
	};

	self.refresh = function() {
		self.fill(true);
	};

	if (toolkit.Environment.isAndroid) {
		var tableoptions = toolkit.Table.HeaderPull.GetOptions();
		tableoptions.backgroundColor = rules.settings.Styles.Window.backgroundColor;
		tableoptions.color = rules.settings.Styles.Window.color;
		tableoptions.borderBackgroundColor = rules.settings.Styles.Window.color;
		var HeaderPullComplete = toolkit.Table.HeaderPull.Create(tableoptions, table, self.refresh);
	}

	return self;
}

function NewCategoryView() {
	var self = Ti.UI.createView();

	var menuBarItems = ['Ultim\'ora', 'Calciomercato', 'Rassegna stampa'];

	var menuBarStyle = toolkit.Controls.Selector.GetOptions();
	menuBarStyle.color = '#fff';
	menuBarStyle.backgroundColor = rules.settings.Styles.Window.backgroundColor;
	menuBarStyle.colorSelectedColor = '#fff';
	menuBarStyle.backgroundSelectedColor = rules.settings.Styles.TableRow.backgroundColor;
	menuBarStyle.height = 40;
	menuBarStyle.font = {
		fontSize : 12
	};
	menuBarStyle.baselineSmall = false;
	menuBarStyle.changeColorTextOnSelect = false;

	if (toolkit.Environment.isAndroid && toolkit.Environment.isTablet) {
		menuBarStyle.height = 60;
		menuBarStyle.font = {
			fontSize : 20
		};
	}

	var menuBar = new toolkit.Controls.Selector.Create(menuBarItems, menuBarStyle);
	self.add(menuBar);
	menuBar.addEventListener('toolkit_SelectorSelected', function(e) {
		scrollable.setCurrentPage(menuBar.getIndex());
		self.fill();
	});

	var scrollable = Ti.UI.createScrollableView({
		top : menuBarStyle.height,
		showPagingControl : false
	});
	scrollable.addEventListener('scrollend', function() {
		menuBar.setIndex(scrollable.getCurrentPage());
		self.fill();
	});
	self.add(scrollable);

	var views = [];

	var view;

	view = new getView(rules.settings.News.UltimaOra.rss);
	if (toolkit.Environment.isAndroid) {
		view.addEventListener('selectedItem', function(e) {
			self.fireEvent('selectedItem', e);
		});
	}
	var ultimaOraView = view;
	views.push(ultimaOraView);

	view = new getView(rules.settings.News.CalcioMercato.rss);
	if (toolkit.Environment.isAndroid) {
		view.addEventListener('selectedItem', function(e) {
			self.fireEvent('selectedItem', e);
		});
	}
	var calcioMercatoView = view;
	views.push(calcioMercatoView);

	view = new getView(rules.settings.News.RassegnaStampa.rss);
	if (toolkit.Environment.isAndroid) {
		view.addEventListener('selectedItem', function(e) {
			self.fireEvent('selectedItem', e);
		});
	}
	var rassegnaStampaView = view;
	views.push(rassegnaStampaView);

	scrollable.views = views;

	var rsslink;
	self.fill = function(force) {
		views[scrollable.getCurrentPage()].fill(force);
	};

	return self;
};

module.exports = NewCategoryView;
