var toolkit = require('/lib/toolkit');
var rules = require('/lib/rules');

function getRow(item) {
	var self = Ti.UI.createTableViewRow({
		className : 'StatisticheGiocatoreRowView',
		backgroundColor : rules.settings.Styles.TableRow.backgroundColor,
		selectedBackgroundColor : rules.settings.Styles.TableRow.selectedBackgroundColor,
		height : 20
	});

	if (toolkit.Environment.isAndroid && toolkit.Environment.isTablet) {
		self.height *= 2;
	}

	self.add(getLabel(item.anno, 5, 55, 10));
	self.add(getLabel(item.squadra, 65, 85, 10));
	self.add(getLabel(item.presenze, 160, 95, 10));
	self.add(getLabel(item.reti, 260, 55, 10));

	return self;
}

function getLabel(text, left, width, fontSize) {
	var self = Ti.UI.createLabel({
		text : text,
		left : left,
		color : 'white',
		width : width,
		textAlign : 'center',
		font : {
			fontSize : fontSize
		}
	});

	if (toolkit.Environment.isAndroid && toolkit.Environment.isTablet) {
		self.left *= 2;
		self.width *= 2;
		self.font = {
			fontSize : self.font.fontSize*2
		};
	}

	return self;
}

function RosaStatisticheView() {
	var self = Ti.UI.createView({
		backgroundColor : rules.settings.Styles.Window.backgroundColor,
		borderWidth : 2,
		borderColor : '#fff'
	});

	var imgLogo = Ti.UI.createImageView({
		width : 100,
		height : 100
	});
	toolkit.Controls.SetIndicator(imgLogo);
	imgLogo.RemoteImage('http://' + rules.settings.Logo.Retina);
	self.add(imgLogo);

	if (toolkit.Environment.isAndroid) {
		imgLogo.width = Ti.Platform.displayCaps.platformWidth / 3;
		imgLogo.height = imgLogo.width;
	}

	var containerView = Ti.UI.createView({
		top : 2,
		right : 2,
		bottom : 2,
		left : 2,
		visible : false,
		layout : 'vertical'
	});
	self.add(containerView);

	var headerView = Ti.UI.createView({
		height : imgLogo.height,
		backgroundColor : rules.settings.Styles.Window.backgroundColor
	});
	containerView.add(headerView);

	var lblName = Ti.UI.createLabel({
		top : 5,
		left : imgLogo.width + 10,
		color : 'white',
		font : {
			fontSize : (imgLogo.width * 0.16)
		}
	});
	headerView.add(lblName);

	var lblRole = Ti.UI.createLabel({
		top : imgLogo.width * 0.30,
		left : lblName.left,
		color : 'white',
		font : {
			fontSize : (imgLogo.width * 0.14)
		}
	});
	headerView.add(lblRole);

	var img = Ti.UI.createImageView({
		left : 0,
		width : imgLogo.width,
		backgroundColor : 'white'
	});
	headerView.add(img);
	toolkit.Controls.SetIndicator(img);

	var tableHeaderView = Ti.UI.createView({
		height : (imgLogo.width * 0.20),
		backgroundColor : rules.settings.Styles.Window.backgroundColor
	});
	containerView.add(tableHeaderView);
	tableHeaderView.add(getLabel('ANNO', 5, 55));
	tableHeaderView.add(getLabel('SQUADRA', 65, 85));
	tableHeaderView.add(getLabel('PRESENZE', 160, 95));
	tableHeaderView.add(getLabel('RETI', 260, 55));

	var table = Ti.UI.createTableView({
		separatorColor : rules.settings.Styles.TableRow.separatorColor,
		backgroundColor : 'transparent'
	});
	containerView.add(table);

	self.fill = function(item) {
		if (!Ti.Network.online) {
			toolkit.Dialog.Error(L('notconnection'));
			return;
		}

		lblName.text = item.nome + ' (' + item.maglietta + ')';
		lblRole.text = item.ruolo;
		img.RemoteImage('http://' + item.link);

		table.data = [];

		for (var i = 0; i < item.statistiche.length; i++) {
			table.appendRow(getRow(item.statistiche[i]));
		}
	};

	self.ShowHide = function(visible) {
		containerView.visible = visible;
	};

	return self;
}

module.exports = RosaStatisticheView;
